@@ -1,5 +1,6 @@
 from __future__ import division
 import sys, os
+from types import FunctionType
 
 import numpy as np
 from matplotlib.pylab import figure
@@ -8,29 +9,32 @@ import matplotlib.pyplot as plt
 import pandas as pd
 import pymc as pm
 import pymc.progressbar as pbar
+import utils
 
 from utils import interpolate_trace
-from itertools import combinations
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
+from collections import OrderedDict
 
-
-def plot_posterior_nodes(nodes, bins=50):
+def plot_posterior_nodes(nodes, bins=50, lb=None, ub=None):
     """Plot interpolated posterior of a list of nodes.
 
     :Arguments:
         nodes : list of pymc.Node's
-            List of pymc.Node's to plot the posterior of
+            List of pymc.Node's to plot the posterior of.
+            These can be found in model.nodes_db.node.ix['param_name']
         bins : int (default=50)
             How many bins to use for computing the histogram.
-
+        lb : float (default is to infer from data)
+            Lower boundary to use for plotting.
+        ub : float (default is to infer from data)
+            Upper boundary to use for plotting.
     """
     figure()
-    lb = min([min(node.trace()[:]) for node in nodes])
-    ub = max([max(node.trace()[:]) for node in nodes])
+    if lb is None:
+        lb = min([min(node.trace()[:]) for node in nodes])
+    if ub is None:
+        ub = max([max(node.trace()[:]) for node in nodes])
+
     x_data = np.linspace(lb, ub, 300)
 
     for node in nodes:
@@ -107,30 +111,6 @@ def group_plot(model, params_to_plot=(), bins=50, samples=5000, save_to=None):
             plt.savefig(os.path.join(save_to, "group_%s.png" % knode_name))
             plt.savefig(os.path.join(save_to, "group_%s.pdf" % knode_name))
 
-def compare_all_pairwise(model):
-    """Perform all pairwise comparisons of dependent parameter
-    distributions (as indicated by depends_on).
-     :Stats generated:
-        * Mean difference
-        * 5th and 95th percentile
-    """
-    raise NotImplementedError, "compare_all_pairwise is currently not functional."
-    from scipy.stats import scoreatpercentile
-
-    print "Parameters\tMean difference\t5%\t95%"
-
-    # Loop through dependent parameters and generate stats
-    for param in model.nodes_db.itervalues():
-        if len(param.group_nodes) < 2:
-            continue
-        # Loop through all pairwise combinations
-        for p0,p1 in combinations(param.group_nodes.values(), 2):
-            diff = p0.trace() - p1.trace()
-            perc_5 = scoreatpercentile(diff, 5)
-            perc_95 = scoreatpercentile(diff, 95)
-            print "%s vs %s\t%.3f\t%.3f\t%.3f" %(p0.__name__, p1.__name__, np.mean(diff), perc_5, perc_95)
-
-
 def plot_all_pairwise(model):
     """Plot all pairwise posteriors to find correlations."""
     import scipy as sp
@@ -237,8 +217,8 @@ def group_cond_diff(hm, node, cond1, cond2, threshold=0):
 
     return pooled_mean, pooled_var, mass_under
 
-def _evaluate_post_pred(sampled_stats, data_stats, evals=None):
-    """Evaluate a summary statistics of sampled sets.
+def post_pred_compare_stats(sampled_stats, data_stats, evals=None):
+    """Evaluate summary statistics of sampled sets.
 
     :Arguments:
         sampled_stats : dict
@@ -256,79 +236,54 @@ def _evaluate_post_pred(sampled_stats, data_stats, evals=None):
         # Generate some default evals
         evals = OrderedDict()
         evals['observed'] = lambda x, y: y
-        evals['credible'] = lambda x, y: (scoreatpercentile(x, 97.5) > y) and (scoreatpercentile(x, 2.5) < y)
-        evals['quantile'] = percentileofscore
-        evals['SEM'] = lambda x, y: (np.mean(x) - y)**2
-        evals['mahalanobis'] = lambda x, y: np.abs(np.mean(x) - y)/np.std(x)
         evals['mean'] = lambda x,y: np.mean(x)
         evals['std'] = lambda x,y: np.std(x)
-        for q in [2.5, 25, 50, 75, 97.5]:
-            key = str(q) + 'q'
-            evals[key] = lambda x, y, q=q: scoreatpercentile(x, q)
+        evals['SEM'] = lambda x, y: (np.mean(x) - y)**2
+        evals['MSE'] = lambda x, y: np.mean((x - y)**2)
+        evals['credible'] = lambda x, y: (scoreatpercentile(x, 97.5) > y) and (scoreatpercentile(x, 2.5) < y)
+        evals['quantile'] = percentileofscore
+        evals['mahalanobis'] = lambda x, y: np.abs(np.mean(x) - y)/np.std(x)
+        #for q in [2.5, 25, 50, 75, 97.5]:
+        #    key = str(q) + 'q'
+        #    evals[key] = lambda x, y, q=q: scoreatpercentile(x, q)
 
     # Evaluate all eval-functions
-    results = pd.DataFrame(index=sampled_stats.keys(), columns=evals.keys() + ['NaN'])
+    results = pd.DataFrame(index=sampled_stats.keys(), columns=evals.keys() + ['NaN'],
+                           dtype=np.float32)
+
     results.index.names = ['stat']
-    for stat_name in sampled_stats.iterkeys():
+    for stat_name in sampled_stats:
         #update NaN column with the no. of NaNs and remove them
         s = sampled_stats[stat_name]
-        results.ix[stat_name]['NaN'] = sum(np.isnan(s))
+        results.ix[stat_name]['NaN'] = sum(pd.isnull(s))
         s = s[np.isfinite(s)]
         if len(s) == 0:
             continue
         #evaluate
         for eval_name, func in evals.iteritems():
             value = func(s, data_stats[stat_name])
-            assert np.isscalar(value), "eval function %s is not returning scalar." % eval_name
             results.ix[stat_name][eval_name] = value
 
-    return results
+    return results.drop('NaN', axis=1)
 
 
-def _post_pred_summary_bottom_node(bottom_node, samples=500, stats=None, plot=False, bins=100, evals=None):
-    """Create posterior predictive check for a single bottom node."""
-    def _calc_stats(data, stats):
-        out = {}
-        for name, func in stats.iteritems():
-            out[name] = func(data)
-        return out
-
-    if stats is None:
-        stats = OrderedDict((('mean', np.mean), ('std', np.std)))
-
-    ############################
-    # Compute stats over data
-    data = bottom_node.value
-    data_stats = _calc_stats(data, stats)
-
-    ###############################################
-    # Initialize posterior sample stats container
-    sampled_stats = {}
-    for name in stats.iterkeys():
-        sampled_stats[name] = np.empty(samples)
+def _post_pred_generate(bottom_node, samples=500, data=None, append_data=False):
+    """Generate posterior predictive data from a single observed node."""
+    datasets = []
 
     ##############################
     # Sample and generate stats
     for sample in range(samples):
         _parents_to_random_posterior_sample(bottom_node)
         # Generate data from bottom node
-        sampled = bottom_node.random()
-        sampled_stat = _calc_stats(sampled, stats)
+        sampled_data = bottom_node.random()
+        if append_data and data is not None:
+            sampled_data = sampled_data.join(data, lsuffix='_sampled')
+        datasets.append(sampled_data)
 
-        # Add it the results container
-        for name, value in sampled_stat.iteritems():
-            sampled_stats[name][sample] = value
+    return datasets
 
-    if plot:
-        from pymc.Matplot import gof_plot
-        for name, value in sampled_stats.iteritems():
-            gof_plot(value, data_stats[name], nbins=bins, name=name, verbose=0)
-
-    result = _evaluate_post_pred(sampled_stats, data_stats, evals=evals)
-
-    return result
-
-def post_pred_check(model, samples=500, bins=100, stats=None, evals=None, plot=False, progress_bar=True):
+def post_pred_gen(model, groupby=None, samples=500, append_data=False, progress_bar=True):
     """Run posterior predictive check on a model.
 
     :Arguments:
@@ -338,38 +293,40 @@ def post_pred_check(model, samples=500, bins=100, stats=None, evals=None, plot=F
     :Optional:
         samples : int
             How many samples to generate for each node.
-        bins : int
-            How many bins to use for computing the histogram.
-        stats : dict
-            User-defined statistics to compute (by default mean and std are computed)
-            and evaluate over the samples.
-            :Example: {'mean': np.mean, 'median': np.median}
-        evals : dict
-            User-defined evaluations of the statistics (by default 95 percentile and SEM).
-            :Example: {'percentile': scoreatpercentile}
-        plot : bool
-            Whether to plot the posterior predictive distributions.
-        progress_bar: bool
-            Display progress bar while sampling.
-
+        groupby : list
+            Alternative grouping of the data. If not supplied, uses splitting
+            of the model (as provided by depends_on).
+        append_data : bool (default=False)
+            Whether to append the observed data of each node to the replicatons.
+        progress_bar : bool (default=True)
+            Display progress bar
 
     :Returns:
-        Hierarchical pandas.DataFrame with the different statistics.
+        Hierarchical pandas.DataFrame with multiple sampled RT data sets.
+        1st level: wfpt node
+        2nd level: posterior predictive sample
+        3rd level: original data index
 
+    :See also:
+        post_pred_stats
     """
-    import pandas as pd
     results = {}
 
     # Progress bar
     if progress_bar:
-        n_iter = len(model.get_observeds()) * model.num_subjs
-        bar = pbar.ProgressBar(n_iter)
+        n_iter = len(model.get_observeds())
+        bar = pbar.progress_bar(n_iter)
         bar_iter = 0
     else:
         print "Sampling..."
 
-    for name, obs_descr in model.iter_observeds():
-        node = obs_descr['node']
+    if groupby is None:
+        iter_data = ((name, model.data.ix[obs['node'].value.index]) for name, obs in model.iter_observeds())
+    else:
+        iter_data = model.data.groupby(groupby)
+
+    for name, data in iter_data:
+        node = model.get_data_nodes(data.index)
 
         if progress_bar:
             bar_iter += 1
@@ -378,12 +335,87 @@ def post_pred_check(model, samples=500, bins=100, stats=None, evals=None, plot=F
         if node is None or not hasattr(node, 'random'):
             continue # Skip
 
-        results[name] = _post_pred_summary_bottom_node(node, samples=samples, bins=bins, evals=evals, stats=stats, plot=plot)
-        if progress_bar:
-            bar.animate(n_iter)
+        ##############################
+        # Sample and generate stats
+        datasets = _post_pred_generate(node, samples=samples, data=data, append_data=append_data)
+        results[name] = pd.concat(datasets, names=['sample'], keys=range(len(datasets)))
+
+    if progress_bar:
+        bar_iter += 1
+        bar.update(bar_iter)
 
     return pd.concat(results, names=['node'])
 
+
+def post_pred_stats(data, sim_datasets, stats=None, plot=False, bins=100, evals=None, call_compare=True):
+    """Calculate a set of summary statistics over posterior predictives.
+
+    :Arguments:
+        data : pandas.Series
+
+        sim_data : pandas.Series
+
+    :Optional:
+        bins : int
+            How many bins to use for computing the histogram.
+        stats : dict or function
+            User-defined statistics to compute (by default mean and std are computed)
+            and evaluate over the samples.
+            :Example:
+              * {'mean': np.mean, 'median': np.median}
+              * lambda x: np.mean(x)
+        evals : dict
+            User-defined evaluations of the statistics (by default 95 percentile and SEM).
+            :Example: {'percentile': scoreatpercentile}
+        plot : bool
+            Whether to plot the posterior predictive distributions.
+        progress_bar : bool
+            Display progress bar while sampling.
+        field : string
+            Which column name to run the stats on
+        call_com,pare : bool (default=True)
+            Whether to call post_pred_compare_stats. If False, return stats directly.
+    """
+
+    def _calc_stats(data, stats):
+        out = {}
+        for name, func in stats.iteritems():
+            out[name] = func(data)
+        return out
+
+    if stats is None:
+        stats = OrderedDict((('mean', np.mean), ('std', np.std)))
+    if isinstance(stats, FunctionType):
+        stats = OrderedDict((('stat', stats),))
+
+    data_stats = _calc_stats(data, stats)
+
+    ###############################################
+    # Initialize posterior sample stats container
+    samples = len(sim_datasets)
+    sampled_stats = {}
+    sampled_stats = pd.DataFrame(index=sim_datasets.index.droplevel(2).unique(),
+                                 columns=stats.keys(),
+                                 dtype=np.float32)
+
+    for i, sim_dataset in sim_datasets.groupby(level=(0, 1)):
+        sampled_stat = _calc_stats(sim_dataset.values, stats)
+
+        # Add it to the results container
+        for name, value in sampled_stat.iteritems():
+            sampled_stats[name][i] = value
+
+    if plot:
+        from pymc.Matplot import gof_plot
+        for name, value in sampled_stats.iteritems():
+            gof_plot(value, data_stats[name], nbins=bins, name=name, verbose=0)
+
+    if call_compare:
+        return post_pred_compare_stats(sampled_stats, data_stats, evals=evals)
+    else:
+        return sampled_stats
+
+
 def _parents_to_random_posterior_sample(bottom_node, pos=None):
     """Walks through parents and sets them to pos sample."""
     for i, parent in enumerate(bottom_node.extended_parents):
@@ -398,13 +430,16 @@ def _parents_to_random_posterior_sample(bottom_node, pos=None):
         parent.value = parent.trace()[pos]
 
 
-def _post_pred_bottom_node(bottom_node, value_range, samples=10, bins=100, axis=None):
+def _plot_posterior_pdf_node(bottom_node, axis, value_range=None, samples=10, bins=100):
     """Calculate posterior predictive for a certain bottom node.
 
     :Arguments:
         bottom_node : pymc.stochastic
             Bottom node to compute posterior over.
 
+        axis : matplotlib.axis
+            Axis to plot into.
+
         value_range : numpy.ndarray
             Range over which to evaluate the likelihood.
 
@@ -415,10 +450,12 @@ def _post_pred_bottom_node(bottom_node, value_range, samples=10, bins=100, axis=
         bins : int (default=100)
             Number of bins to compute histogram over.
 
-        axis : matplotlib.axis (default=None)
-            If provided, will plot into axis.
     """
 
+    if value_range is None:
+        # Infer from data by finding the min and max from the nodes
+        raise NotImplementedError, "value_range keyword argument must be supplied."
+
     like = np.empty((samples, len(value_range)), dtype=np.float32)
     for sample in range(samples):
         _parents_to_random_posterior_sample(bottom_node)
@@ -432,23 +469,20 @@ def _post_pred_bottom_node(bottom_node, value_range, samples=10, bins=100, axis=
         print "WARNING! %s threw FloatingPointError over std computation. Setting to 0 and continuing." % bottom_node.__name__
         y_std = np.zeros_like(y)
 
+    # Plot pp
+    axis.plot(value_range, y, label='post pred', color='b')
+    axis.fill_between(value_range, y-y_std, y+y_std, color='b', alpha=.8)
 
-    if axis is not None:
-        # Plot pp
-        axis.plot(value_range, y, label='post pred', color='b')
-        axis.fill_between(value_range, y-y_std, y+y_std, color='b', alpha=.8)
+    # Plot data
+    if len(bottom_node.value) != 0:
+        axis.hist(bottom_node.value.values, normed=True, color='r',
+                  range=(value_range[0], value_range[-1]), label='data',
+                  bins=bins, histtype='step', lw=2.)
 
-        # Plot data
-        if len(bottom_node.value) != 0:
-            axis.hist(bottom_node.value, normed=True, color='r',
-                      range=(value_range[0], value_range[-1]), label='data',
-                      bins=bins, histtype='step', lw=2.)
+    axis.set_ylim(bottom=0) # Likelihood and histogram can only be positive
 
-        axis.set_ylim(bottom=0) # Likelihood and histogram can only be positive
-
-    return (y, y_std) #, hist, ranges)
-
-def plot_posterior_predictive(model, value_range=None, samples=10, columns=3, bins=100, savefig=False, path=None, figsize=(8,6)):
+def plot_posterior_predictive(model, plot_func=None, required_method='pdf', columns=None, save=False, path=None,
+                              figsize=(8,6), format='png', **kwargs):
     """Plot the posterior predictive distribution of a kabuki hierarchical model.
 
     :Arguments:
@@ -471,46 +505,87 @@ def plot_posterior_predictive(model, value_range=None, samples=10, columns=3, bi
         bins : int (default=100)
             How many bins to compute the data histogram over.
 
-        savefig : bool (default=False)
+        figsize : (int, int) (default=(8, 6))
+
+        save : bool (default=False)
             Whether to save the figure to a file.
 
         path : str (default=None)
             Save figure into directory prefix
 
+        format : str or list of strings
+            Save figure to a image file of type 'format'. If more then one format is
+            givven them multiple files are created
+
+        plot_func : function (default=_plot_posterior_pdf_node)
+            Plotting function to use for each observed node
+            (see default function for an example).
+
     :Note:
 
         This function changes the current value and logp of the nodes.
 
     """
 
-    if value_range is None:
-        # Infer from data by finding the min and max from the nodes
-        raise NotImplementedError, "value_range keyword argument must be supplied."
+    if plot_func is None:
+        plot_func = _plot_posterior_pdf_node
 
     observeds = model.get_observeds()
 
+    if columns is None:
+        # If there are less than 3 items to plot per figure,
+        # only use as many columns as there are items.
+        max_items = max([len(i[1]) for i in
+                         observeds.groupby('tag').groups.iteritems()])
+        columns = min(3, max_items)
 
+    # Plot different conditions (new figure for each)
     for tag, nodes in observeds.groupby('tag'):
         fig = plt.figure(figsize=figsize)
-        fig.suptitle(tag, fontsize=12)
+        fig.suptitle(utils.pretty_tag(tag), fontsize=12)
         fig.subplots_adjust(top=0.9, hspace=.4, wspace=.3)
 
+        # Plot individual subjects (if present)
         for subj_i, (node_name, bottom_node) in enumerate(nodes.iterrows()):
-            if not hasattr(bottom_node['node'], 'pdf'):
-                continue # skip nodes that do not define pdf function
+            if not hasattr(bottom_node['node'], required_method):
+                continue # skip nodes that do not define the required_method
 
             ax = fig.add_subplot(np.ceil(len(nodes)/columns), columns, subj_i+1)
             if 'subj_idx' in bottom_node:
                 ax.set_title(str(bottom_node['subj_idx']))
-            _post_pred_bottom_node(bottom_node['node'], value_range,
-                                   axis=ax, bins=bins)
 
-        if savefig:
-            tag_str = '.'.join(tag)
-            if path is not None:
-                fig.savefig(os.path.join(path, tag_str) + '.svg', format='svg')
-                fig.savefig(os.path.join(path, tag_str) + '.png', format='png')
-            else:
-                fig.savefig(tag_str + '.svg', format='svg')
-                fig.savefig(tag_str + '.png', format='png')
+            plot_func(bottom_node['node'], ax, **kwargs)
 
+        # Save figure if necessary
+        if save:
+            fname = 'ppq_' + '.'.join(tag)
+            if path is None:
+                path = '.'
+            if isinstance(format, str):
+                format = [format]
+            [fig.savefig('%s.%s' % (os.path.join(path, fname), x), format=x) for x in format]
+
+def geweke_problems(model, fname=None, **kwargs):
+    """
+    return a list of nodes who were detected as problemtic according to the geweke test
+    Input:
+        fname : string (deafult - None)
+            Save result to file named fname
+        kwargs : keywords argument passed to the geweke function
+    """
+
+    #search for geweke problems
+    g = pm.geweke(model.mc)
+    problems = []
+    for node, output in g.iteritems():
+        values = np.array(output)[:,1]
+        if np.any(np.abs(values) > 2):
+            problems.append(node)
+
+    #write results to file if needed
+    if fname is not None:
+        with open(fname, 'w') as f:
+            for node in problems:
+                f.write(node)
+
+    return problems
