@@ -2,6 +2,7 @@
 from __future__ import division
 from copy import copy
 import pickle
+import sys
 
 import numpy as np
 from scipy.optimize import fmin_powell, fmin
@@ -14,9 +15,27 @@ import warnings
 
 from kabuki.utils import flatten
 
+class LnProb(object):
+    def __init__(self, model):
+        self.model = model
+
+    def lnprob(self, vals): # vals is a vector of parameter values to try
+        # Set each random variable of the pymc model to the value
+        # suggested by emcee
+        try:
+            for val, (name, stoch) in zip(vals, self.model.iter_stochastics()):
+                stoch['node'].set_value(val)
+            logp = self.model.mc.logp
+            return logp
+        except pm.ZeroProbability:
+            return -np.inf
+
+    def __call__(self, *args, **kwargs):
+        return self.lnprob(*args, **kwargs)
 
 class Knode(object):
-    def __init__(self, pymc_node, name, depends=(), col_name='', subj=False, hidden=False, **kwargs):
+    def __init__(self, pymc_node, name, depends=(), col_name='',
+                 subj=False, hidden=False, pass_dataframe=True, **kwargs):
         self.pymc_node = pymc_node
         self.name = name
         self.kwargs = kwargs
@@ -28,6 +47,8 @@ class Knode(object):
         self.nodes = OrderedDict()
         self.hidden = hidden
 
+        self.pass_dataframe = pass_dataframe
+
         #create self.parents
         self.parents = {}
         for (name, value) in self.kwargs.iteritems():
@@ -116,7 +137,10 @@ class Knode(object):
 
             #get value for observed node
             if self.observed:
-                kwargs['value'] = grouped_data[self.col_name] #.to_records(index=False)
+                if self.pass_dataframe:
+                    kwargs['value'] = grouped_data[self.col_name] #.to_records(index=False)
+                else:
+                    kwargs['value'] = grouped_data[self.col_name].values #.to_records(index=False)
 
             # Deterministic nodes require a parent argument that is a
             # dict mapping parent names to parent nodes. Knode wraps
@@ -131,6 +155,9 @@ class Knode(object):
                     kwargs.pop(name)
                 kwargs['parents'] = parents_dict
 
+                if self.observed:
+                    kwargs['parents']['value'] = kwargs['value']
+
 
             # Deterministic nodes require a doc kwarg, we don't really
             # need that so if its not supplied, just use the name
@@ -313,6 +340,9 @@ class Hierarchical(object):
             self._num_subjs = 1
 
         self.num_subjs = self._num_subjs
+        self.sampled = False
+        self.dbname = 'ram'
+        self.db = None
 
         self._setup_model()
 
@@ -333,17 +363,19 @@ class Hierarchical(object):
         d['nodes_db'] = deepcopy(d['nodes_db'].drop('node', axis=1))
         d['depends'] = dict(d['depends'])
         #d['model_type'] = self.__class__
-        d['db'] = self.mc.db.__name__
 
-        dbname = d['mc'].db.__name__
-	if (dbname == 'ram'):
-            raise ValueError("db is 'ram'. Saving a model requires a database on disk.")
-	elif (dbname == 'pickle'):
-            d['dbname'] = d['mc'].db.filename
-	elif (dbname == 'txt'):
-            d['dbname'] = d['mc'].db._directory
-	else: # hdf5, sqlite
-            d['dbname'] = d['mc'].db.dbname
+        if self.sampled:
+            d['db'] = self.mc.db.__name__
+
+            dbname = d['mc'].db.__name__
+            if (dbname == 'ram'):
+                    raise ValueError("db is 'ram'. Saving a model requires a database on disk.")
+            elif (dbname == 'pickle'):
+                    d['dbname'] = d['mc'].db.filename
+            elif (dbname == 'txt'):
+                    d['dbname'] = d['mc'].db._directory
+            else: # hdf5, sqlite
+                    d['dbname'] = d['mc'].db.dbname
 
         del d['mc']
         del d['knodes']
@@ -354,7 +386,12 @@ class Hierarchical(object):
         self.__dict__.update(d)
         self._setup_model()
         self.create_model()
-        self.load_db(d['dbname'], db=d['db'])
+
+        if self.sampled:
+            self.load_db(d['dbname'], db=d['db'])
+            self.gen_stats()
+        else:
+            self.mcmc()
 
     def save(self, fname):
         """Save model to file.
@@ -405,6 +442,16 @@ class Hierarchical(object):
     def create_nodes_db(self):
         self.nodes_db = pd.concat([knode.nodes_db for knode in self.knodes])
 
+    def draw_from_prior(self):
+        non_zero = False
+        while non_zero:
+            try:
+                self.mc.draw_from_prior()
+                self.mc.logp
+                non_zero = True
+            except pm.ZeroProbability:
+                non_zero = False
+
 
     def map(self, runs=2, warn_crit=5, method='fmin_powell', **kwargs):
         """
@@ -434,16 +481,15 @@ class Hierarchical(object):
             raise NotImplementedError("""Sorry, This method is not yet implemented for group models.
             you might consider using the approximate_map method""")
 
-
         maps = []
 
         for i in range(runs):
             # (re)create nodes to get new initival values.
             #nodes are not created for the first iteration if they already exist
             if i != 0:
-                self.create_model()
+                self.draw_from_prior()
 
-            m = pm.MAP(self.nodes_db.node.values)
+            m = pm.MAP(self.nodes_db.node)
             m.fit(method, **kwargs)
             print m.logp
             maps.append(m)
@@ -461,7 +507,7 @@ class Hierarchical(object):
 
         # Set values of nodes
         for max_node in max_map.stochastics:
-            self.nodes_db.ix[max_node.__name__]['node'].value = max_node.value
+            self.nodes_db.node.ix[max_node.__name__].set_value(max_node.value)
 
         return max_map
 
@@ -485,6 +531,85 @@ class Hierarchical(object):
     def pre_sample(self):
         pass
 
+    def sample_emcee(self, nwalkers=500, samples=10, dispersion=.1, burn=5, thin=1, stretch_width=2., anneal_stretch=True, pool=None):
+        import emcee
+        import pymc.progressbar as pbar
+
+        # This is the likelihood function for emcee
+        lnprob = LnProb(self)
+
+        # init
+        self.mcmc()
+
+        # get current values
+        stochs = self.get_stochastics()
+        start = [node_descr['node'].value for name, node_descr in stochs.iterrows()]
+        ndim = len(start)
+
+        def init_from_priors():
+            p0 = np.empty((nwalkers, ndim))
+            i = 0
+            while i != nwalkers:
+                self.mc.draw_from_prior()
+                try:
+                    self.mc.logp
+                    p0[i, :] = [node_descr['node'].value for name, node_descr in stochs.iterrows()]
+                    i += 1
+                except pm.ZeroProbability:
+                    continue
+            return p0
+
+        if hasattr(self, 'emcee_dispersions'):
+            scale = np.empty_like(start)
+            for i, (name, node_descr) in enumerate(stochs.iterrows()):
+                knode_name = node_descr['knode_name'].replace('_subj', '')
+                scale[i] = self.emcee_dispersions.get(knode_name, 0.1)
+        else:
+            scale = 0.1
+
+        p0 = np.random.randn(ndim * nwalkers).reshape((nwalkers, ndim)) * scale * dispersion + start
+        #p0 = init_from_priors()
+
+        # instantiate sampler passing in the pymc likelihood function
+        sampler = emcee.EnsembleSampler(nwalkers, ndim, lnprob, a=stretch_width, pool=pool)
+
+        bar = pbar.progress_bar(burn + samples)
+        i = 0
+
+        annealing = np.linspace(stretch_width, 2, burn)
+        sys.stdout.flush()
+
+        for pos, prob, state in sampler.sample(p0, iterations=burn):
+            if anneal_stretch:
+                sampler.a = annealing[i]
+            i += 1
+            bar.update(i)
+
+        #print("\nMean acceptance fraction during burn-in: {}".format(np.mean(sampler.acceptance_fraction)))
+        sampler.reset()
+
+        # sample
+        try:
+            for p, lnprob, lnlike in sampler.sample(pos,
+                                                    iterations=samples,
+                                                    thin=thin):
+                i += 1
+                bar.update(i)
+        except KeyboardInterrupt:
+            pass
+        finally:
+            print("\nMean acceptance fraction during sampling: {}".format(np.mean(sampler.acceptance_fraction)))
+            # restore state
+            for val, (name, node_descr) in zip(start, stochs.iterrows()):
+                node_descr['node'].set_value(val)
+
+            # Save samples back to pymc model
+            self.mc.sample(1, progress_bar=False) # This call is to set up the chains
+            for pos, (name, node) in enumerate(stochs.iterrows()):
+                node['node'].trace._trace[0] = sampler.flatchain[:, pos]
+
+            return sampler
+
     def sample(self, *args, **kwargs):
         """Sample from posterior.
 
@@ -509,6 +634,8 @@ class Hierarchical(object):
         # sample
         self.mc.sample(*args, **kwargs)
 
+        self.sampled = True
+
         return self.mc
 
     @property
@@ -754,9 +881,24 @@ class Hierarchical(object):
         """
         return pd.DataFrame({i.__name__: i.trace() for i in self.get_stochastics().node})
 
+    def get_data_nodes(self, idx):
+        data_nodes = []
+        for name, node_descr in self.iter_observeds():
+            node = node_descr['node']
+            if set(idx).issubset(set(node.value.index)):
+                data_nodes.append(node)
+
+        if len(data_nodes) != 1:
+            raise NotImplementedError("Supply a grouping so that at most 1 observed node codes for each group.")
+
+        return data_nodes[0]
+
+    def __getitem__(self, name):
+        return self.nodes_db.ix[name]['node']
+
     @property
     def values(self):
-        return {name: node['node'].value[()] for (name, node) in self.iter_non_observeds()}
+        return {name: node['node'].value[()] for (name, node) in self.iter_non_observeds() if node['node'].value.shape == ()}
 
     def set_values(self, new_values):
         """
@@ -765,7 +907,7 @@ class Hierarchical(object):
             new_values <dict> - dictionary of the format {'node_name1': new_value1, ...}
         """
         for (name, value) in new_values.iteritems():
-            self.nodes_db.ix[name]['node'].value = value
+            self.nodes_db.ix[name]['node'].set_value(value)
 
     def find_starting_values(self):
         """Find good starting values for the different parameters by
@@ -790,7 +932,7 @@ class Hierarchical(object):
         # define function to be optimized
         def opt(values):
             for value, node in zip(values, optimize_nodes):
-                node.value = value
+                node.set_value(value)
             try:
                 logp_optimize = [node.logp for node in optimize_nodes]
                 logp_evaluate = [node.logp for node in evaluate_nodes]
