@@ -22,7 +22,7 @@ class TestHierarchicalBreakDown(unittest.TestCase):
         self.models, self.params = create_test_models()
 
         #run models
-        sample_from_models(self.models, n_iter=200)
+        sample_from_models(self.models, n_iter=500)
 
     def runTest(self):
         pass
@@ -30,14 +30,19 @@ class TestHierarchicalBreakDown(unittest.TestCase):
     def test_map(self):
         np.random.seed(123)
         for model in self.models:
-            if model.is_group_model:
-                model.approximate_map()
-            else:
-                model.map(runs=2)
+            model.find_starting_values()
+
+    def test_emcee(self):
+        self.models, self.params = create_test_models()
+        np.random.seed(123)
+        for model in self.models:
+            model.find_starting_values()
+            model.sample_emcee()
+
 
     def test_dic_info(self):
         for model in self.models:
-            model.dic_info()
+            model.dic_info
 
     def test_print_stats(self):
         for model in self.models:
@@ -168,3 +173,24 @@ class TestEstimation(unittest.TestCase):
 
         # looping for each condition (i.e. twice)
         self.assertEqual(counter, subjs*2)
+
+
+class TestConcatenate(unittest.TestCase):
+    def test_concat(self):
+        n_subj = 5
+        data, params = kabuki.generate.gen_rand_data(gen_func_df, {'A':{'loc':0, 'scale':1}, 'B': {'loc':0, 'scale':1}}, subjs=n_subj)
+        data = pd.DataFrame(data)
+
+        models = []
+        for i in range(4):
+            m = HNodeSimple(data)
+            m.sample(100, burn=0, db='pickle', dbname='test_%d'%i)
+            models.append(m)
+
+        super_model = kabuki.utils.concat_models(models)
+        stochs = super_model.get_stochastics()
+        for stoch in stochs.node:
+            self.assertEqual(len(stoch.trace[:]), 100*4)
+
+        for i in range(4):
+            os.remove('test_%d'%i)
